/*
 * Copyright (c) 2001 World Wide Web Consortium,
 * (Massachusetts Institute of Technology, Institut National de
 * Recherche en Informatique et en Automatique, Keio University). All
 * Rights Reserved. This program is distributed under the W3C's Software
 * Intellectual Property License. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.
 * See W3C License http://www.w3.org/Consortium/Legal/ for more details.
 */

package org.w3c.dom.contentModel;

import org.w3c.dom.nsElement;

/**
 * <code>CMModel</code> is an abstract object that could map to a DTD, an XML 
 * Schema, a database schema, etc. It's a generalized content model object, 
 * that has both an internal and external subset. The internal subset would 
 * always exist, even if empty, with the external subset (if present) being 
 * represented as a link to one or more <code>CMExternalModel</code>s. It is 
 * possible, however, that none of these <code>CMExternalModel</code>s are 
 * active.
 * <p>See also the <a href='http://www.w3.org/TR/2001/WD-DOM-Level-3-CMLS-20010209'>Document Object Model (DOM) Level 3 Content Models and Load
and Save Specification</a>.
 */
public interface CMModel extends CMNode {
    /**
     * True if this content model defines the document structure in terms of 
     * namespaces and local names; false if the document structure is 
     * defined only in terms of <code>QNames</code>.
     */
    public boolean getIsNamespaceAware();

    /**
     * The root element declaration for the content model. 
     */
    public ElementDeclaration getRootElementDecl();

    /**
     * Location of the document describing the content model defined in this 
     * CMModel.
     * @return This method returns a DOMString defining the absolute location 
     *   from which this document is retrieved including the document name.
     */
    public String getLocation();

    /**
     * Determines namespace of <code>CMModel</code>.
     * @return Namespace of <code>CMModel</code>.
     */
    public nsElement getCMNamespace();

    /**
     * Returns <code>CMNode</code> list of all the constituent nodes in the 
     * content model. 
     * @return List of all <code>CMNodes</code> of the content model.
     */
    public CMNamedNodeMap getCMNodes();

    /**
     * Removes the specified<code>CMNode</code>.
     * @param node <code>CMNode</code> to be removed.
     * @return <code>Success or failure.</code>.
     */
    public boolean removeNode(CMNode node);

    /**
     * Insert <code>CMNode</code>.
     * @param newNode <code>CMNode</code> to be inserted.
     * @param refNode <code>CMNode</code> to be inserted before.
     * @return <code>Success or failure.</code>.
     */
    public boolean insertBefore(CMNode newNode, 
                                CMNode refNode);

    /**
     * Determines if a CMModel and CMExternalModel itself is valid, i.e., 
     * confirming that it's well-formed and valid per its own formal 
     * grammar. Note that within a CMModel, a pointer to a CMExternalModel 
     * can exist.
     * @return Is the CM valid?
     */
    public boolean validate();

}
