/*
 * Copyright (c) 2001 World Wide Web Consortium,
 * (Massachusetts Institute of Technology, Institut National de
 * Recherche en Informatique et en Automatique, Keio University). All
 * Rights Reserved. This program is distributed under the W3C's Software
 * Intellectual Property License. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.
 * See W3C License http://www.w3.org/Consortium/Legal/ for more details.
 */

package org.w3c.dom.contentModel;

import org.w3c.dom.CMExternalModel *;

/**
 * This interface extends the <code>Document</code> interface with additional 
 * methods for both document and CM editing.
 * <p>See also the <a href='http://www.w3.org/TR/2001/WD-DOM-Level-3-CMLS-20010209'>Document Object Model (DOM) Level 3 Content Models and Load
and Save Specification</a>.
 */
public interface DocumentCM extends Document {
    /**
     * Determines number of <code>CMExternalModel</code>s associated with the 
     * document. Only one <code>CMModel</code> can be associated with the 
     * document, but it may point to a list of <code>CMExternalModel</code>s.
     * @return Non-negative number of external CM objects.
     */
    public int numCMs();

    /**
     * Find the sole <code>CMModel</code> of a document. Only one 
     * <code>CMModel</code> may be associated with the document.
     * @return <code>CMModel</code>.
     */
    public CMModel getInternalCM();

    /**
     * Obtains a list of <code>CMExternalModel</code>s associated with a 
     * document from the <code>CMModel</code>. This list arises when 
     * <code>addCM()</code> is invoked.
     * @return A list of <code>CMExternalModel</code>s associated with a 
     *   document.
     */
    public CMExternalModel * getCMs();

    /**
     * Find the active <code>CMExternalModel</code> for a document.
     * @return <code>CMModel</code> with a pointer to the active 
     *   <code>CMExternalModel</code> of document.
     */
    public CMModel getActiveCM();

    /**
     * Associate a <code>CMModel</code> with a document. Can be invoked 
     * multiple times to result in a list of <code>CMExternalModel</code>s. 
     * Note that only one sole internal <code>CMModel</code> is associated 
     * with the document, however, and that only one of the possible list of 
     * <code>CMExternalModel</code>s is active at any one time.
     * @param cm CM to be associated with the document.
     */
    public void addCM(CMModel cm);

    /**
     * Removes a CM associated with a document; actually removes a 
     * <code>CMExternalModel</code>. Can be invoked multiple times to remove 
     * a number of these in the list of <code>CMExternalModel</code>s.
     * @param cm CM to be removed.
     */
    public void removeCM(CMModel cm);

    /**
     * Make the given <code>CMModel</code> active. Note that if a user wants 
     * to activate one CM to get default attribute values and then activate 
     * another to do validation, a user can do that; however, only one CM is 
     * active at a time.
     * @param cm CM to be active for the document. The <code>CMModel</code> 
     *   points to a list of <code>CMExternalModel</code>s; with this call, 
     *   only the specified CM will be active.
     * @return True if the <code>CMModel</code> has already been associated 
     *   with the document using <code>addCM()</code>; false if not.
     */
    public boolean activateCM(CMModel cm);

}
